import * as i0 from '@angular/core';
import { EventEmitter, forwardRef, Component, Output, Input, HostListener, NgModule } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { BehaviorSubject, Subject } from 'rxjs';
import { takeUntil, filter, debounceTime, skip } from 'rxjs/operators';
import { loadScript, loadCss } from '@cnblogs/load-script-css';
import * as i1 from '@angular/common';
import { CommonModule } from '@angular/common';

const _window = window;
const MaxWaitSeconds = 60;
const waitUntilObjectLoaded = (objectKey) => {
    if (!objectKey)
        return Promise.resolve(false);
    let waitTimes = 0;
    const wait = (resolve) => {
        if (_window[objectKey]) {
            resolve(true);
            return;
        }
        waitTimes++;
        // 等待超时了
        if (waitTimes * 200 > MaxWaitSeconds * 1000) {
            resolve(false);
            return;
        }
        setTimeout(() => {
            wait(resolve);
        }, 200);
    };
    return new Promise(resolve => wait(resolve));
};
const waitUntilEditormdLoaded = () => waitUntilObjectLoaded('editormd');
const waitUntilJQueryLoaded = () => waitUntilObjectLoaded('jQuery');
const waitUntilMathJaxLoaded = () => {
    return waitUntilObjectLoaded('MathJax');
};

const defaultToolbarIcons = [
    'undo',
    'redo',
    '|',
    'bold',
    'del',
    'italic',
    'quote',
    'ucwords',
    'uppercase',
    'lowercase',
    '|',
    'h1',
    'h2',
    'h3',
    'h4',
    'h5',
    'h6',
    '|',
    'list-ul',
    'list-ol',
    'hr',
    'task-lists',
    '|',
    'uploadImage',
    'link',
    'cnb-insert-collapsible-code-block',
    'code',
    'table',
    'datetime',
    'html-entities',
    'pagebreak',
    '|',
    'cnb-search-replace',
    'goto-line',
    'preview',
    'watch',
    'fullscreen',
];

var TaskListsPlugin;
(function (TaskListsPlugin) {
    const svgIcon = `<svg aria-hidden="true" viewBox="0 0 16 16" style="fill: currentColor;" class="icon">
    <path fill-rule="evenodd" d="M2.5 2.75a.25.25 0 01.25-.25h10.5a.25.25 0 01.25.25v10.5a.25.25 0 01-.25.25H2.75a.25.25 0 01-.25-.25V2.75zM2.75 1A1.75 1.75 0 001 2.75v10.5c0 .966.784 1.75 1.75 1.75h10.5A1.75 1.75 0 0015 13.25V2.75A1.75 1.75 0 0013.25 1H2.75zm9.03 5.28a.75.75 0 00-1.06-1.06L6.75 9.19 5.28 7.72a.75.75 0 00-1.06 1.06l2 2a.75.75 0 001.06 0l4.5-4.5z"></path>
</svg>`;
    TaskListsPlugin.toolbarIcon = `<a href="javascript:;" title="插入任务列表"><i class="fa" name="task-lists">${svgIcon}</i></a>`;
    TaskListsPlugin.insertTaskLists = (editor) => {
        const { cm } = editor;
        let selection = cm.getSelection() ?? '';
        const cursor = cm.getCursor();
        let { line: curLine, ch } = cursor;
        const preLine = curLine > 0 ? curLine - 1 : 0;
        const preLineContent = cm.getLine(preLine) ?? '';
        const nextLine = curLine + 1;
        let curLineContent = cm.getLine(curLine) ?? '';
        const nextLineContent = cm.getLine(nextLine) ?? '';
        const mark = `- [ ] `;
        const lastLine = cm.lastLine();
        const isCurLineHasContent = curLineContent.length > 0;
        const isNextLineHasContent = nextLine <= lastLine && nextLineContent?.length > 0;
        const isPreLineHasContent = curLine > 0 && preLineContent?.length > 0;
        let content;
        if (curLineContent.startsWith(mark) && selection) {
            cm.setCursor(curLine, 0);
            selection = '';
        }
        if (selection) {
            cm.setCursor(curLine, curLineContent.length);
            content = `${ch > 0 ? '\n\n' : isPreLineHasContent ? '\n' : ''}${mark}${selection}${isNextLineHasContent ? '\n' : ''}`;
        }
        else {
            cm.setCursor(curLine, curLineContent.length);
            content = `${isCurLineHasContent && !curLineContent.startsWith(mark)
                ? '\n\n'
                : isPreLineHasContent || curLineContent.startsWith(mark)
                    ? '\n'
                    : ''}${mark}${isNextLineHasContent ? '\n' : ''}`;
        }
        cm.replaceSelection(content);
        if (content.endsWith('\n')) {
            curLine = cm.getCursor().line - 1;
            cm.setCursor(curLine, cm.getLine(curLine)?.length);
        }
        cm.focus();
    };
})(TaskListsPlugin || (TaskListsPlugin = {}));

const componentInstances = {};
class EditormdComponent {
    get disabled() {
        return this._disabled;
    }
    set disabled(value) {
        this._disabled = value;
        this.setDisabledState(this._disabled);
    }
    get highlightCodeOption() {
        return this._highlightCodeOption;
    }
    /**
     * 设置实时预览时代码高亮配置
     *
     * `enable`: default is `false`
     *
     *  `highlightCode`: default:
     *  ```js
     *  loadScript(
     *     'prettify-js',
     *     `${this.libPath}lib/prettify.min.js`
     *   ).then(() => {
     *     this._editor.previewCodeHighlight();
     *   });
     *  ```
     */
    set highlightCodeOption(value) {
        this._highlightCodeOption.enable = value.enable;
        if (!value.highlightCode) {
            value.highlightCode = this._highlightCodeOption.highlightCode;
        }
        this._highlightCodeOption.highlightCode = value.highlightCode;
        this.execEnsureEditormdLoaded(() => {
            this.save();
        });
    }
    /**
     * default value is: `https://cdn.jsdelivr.net/npm/mathjax@2/MathJax.js?config=TeX-AMS-MML_CHTML`
     *
     */
    get mathjaxSrc() {
        return this._mathjaxSrc;
    }
    set mathjaxSrc(src) {
        if (src && src.match(/https?:\/\//)) {
            this._mathjaxSrc = src;
            this.execEnsureEditormdLoaded(() => {
                this.save();
            });
        }
    }
    /**
     * 图片上传方法, 粘贴图片时会使用这个方法上传图片
     *
     */
    set uploadImage(value) {
        this._uploadImage = value;
    }
    get uploadImage() {
        return this._uploadImage;
    }
    set enableSearchReplace(v) {
        if (v !== this._enableSearchReplace) {
            this._enableSearchReplace = v;
        }
    }
    get enableSearchReplace() {
        return this._enableSearchReplace;
    }
    get markdownToHtml() {
        return this._markdownToHtml;
    }
    /**
     * markdown转换为html方法, 实时预览时会使用这个方法渲染markdown
     *
     */
    set markdownToHtml(value) {
        this.execEnsureEditormdLoaded(_ => {
            this._markdownToHtml = value;
            this.save();
        });
    }
    /**
     * 加载中占位符
     *
     */
    get loadingTemplate() {
        return this._loadingTemplate;
    }
    set loadingTemplate(value) {
        this._loadingTemplate = value;
    }
    get height() {
        return this._height;
    }
    /**
     * 设置编辑器高度
     *
     */
    set height(value) {
        if (!value) {
            return;
        }
        this._height = value;
        this.execEnsureEditormdLoaded(_ => {
            this.resize(this._width, this._height);
        });
    }
    get width() {
        return this._width;
    }
    /**
     * 设置编辑器宽度
     *
     */
    set width(value) {
        if (!value) {
            return;
        }
        this._width = value;
        if (!this.initializing$.value) {
            this.resize(this._width, this._height);
        }
    }
    get enableMathjax() {
        return this._enableMathjax;
    }
    /**
     * 设置是否开启数学公式支持
     *
     * @type {boolean}
     */
    set enableMathjax(value) {
        if (value !== this._enableMathjax) {
            this._enableMathjax = value;
            this.execEnsureEditormdLoaded(() => this.save());
        }
    }
    get zIndex() {
        return this._zIndex;
    }
    /**
     * z轴顺序, 默认为1000
     * z轴顺序, 默认为1000
     *
     */
    set zIndex(value) {
        if (value !== undefined) {
            this._zIndex = value;
        }
    }
    get realtimePreview() {
        return this._editor.settings.watch;
    }
    /**
     * 设置是否开启实时预览
     *
     */
    set realtimePreview(value) {
        this.execEnsureEditormdLoaded(_ => {
            if (value) {
                this.openRealtimePreview();
            }
            else {
                this.closeRealtimePreview();
            }
        });
    }
    set placeholder(value) {
        if (value) {
            this._placeholder = value;
            this.execEnsureEditormdLoaded(_ => {
                this._editor.cm.setOption('placeholder', this._placeholder);
            });
        }
    }
    get placeholder() {
        return this._editor.cm.getOption('placeholder');
    }
    get editorState() {
        return (this._editor?.state ?? {
            watching: false,
            loaded: false,
            preview: false,
            fullscreen: false,
        });
    }
    get previewContainerEl() {
        const el = this._editor.previewContainer[0];
        if (!el.id) {
            el.id = 'editormd-preview-container';
        }
        return el;
    }
    get previewFrame() {
        return this._editor.previewFrame;
    }
    get previewWindow() {
        return this._editor.previewWindow;
    }
    get previewDocument() {
        return this._editor.previewDocument;
    }
    get markdownTextarea() {
        return this._editor?.markdownTextarea[0];
    }
    constructor() {
        /**
         * 发生用户输入事件时触发
         *
         */
        this.input = new EventEmitter(true);
        this.toolbarIconSaveClicked = new EventEmitter();
        /**
         * Markdown预览内容渲染完成事件
         * @type {EventEmitter<EditormdComponent>}
         */
        this.previewContentReady = new EventEmitter();
        this.loaded = new EventEmitter();
        /**
         * default value is: `editor.md/`
         *
         */
        this.libPath = 'editor.md/';
        /**
         * default to `true`
         */
        this.lineNumbers = true;
        /**
         * default:
         * ```js
         * {
         *   src: `${this.libPath}lib/jquery.min.js`,
         *   tagId: 'jquery-script',
         * }
         * ```
         *
         */
        this.jqueryOption = {
            src: `${this.libPath}lib/jquery.min.js`,
            tagId: 'jquery-script',
        };
        this.toolbarIcons = [];
        this.appendToolbarIcons = [];
        this.saveIconText = '保存';
        this.editormdVersionSuffix = '1.0.0';
        /**
         * 是否正在初始化
         *
         */
        this.initializing$ = new BehaviorSubject(true);
        this._disabled = false;
        this._highlightCodeOption = {
            enable: false,
            highlightCode: (iframe) => {
                console.warn('using default code highlight');
                loadScript({
                    appendTo: iframe.contentDocument?.head,
                    src: `${this.libPath}lib/prettify.min.js`,
                    id: 'prettify-js',
                }).then(() => this._editor.previewCodeHighlight());
            },
        };
        this._mathjaxSrc = 'https://cdn.jsdelivr.net/npm/mathjax@2/MathJax.js?config=TeX-AMS-MML_CHTML';
        this._uploadImage = () => Promise.resolve('');
        this._enableSearchReplace = true;
        this.id = Date.now();
        this._markdownToHtml = markdown => {
            const rendererOptions = {
                toc: this._editor.settings.toc,
                tocm: this._editor.settings.tocm,
                tocStartLevel: this._editor.settings.tocStartLevel,
                pageBreak: this._editor.settings.pageBreak,
                taskList: this._editor.settings.taskList,
                emoji: this._editor.settings.emoji,
                tex: this._editor.settings.tex,
                atLink: this._editor.settings.atLink, // for @link
                emailLink: this._editor.settings.emailLink, // for mail address auto link
                flowChart: this._editor.settings.flowChart,
                sequenceDiagram: this._editor.settings.sequenceDiagram,
                previewCodeHighlight: false,
            };
            const markedOptions = {
                renderer: editormd.markedRenderer(undefined, rendererOptions),
                gfm: true,
                tables: true,
                breaks: true,
                pedantic: false,
                sanitize: !this._editor.settings.htmlDecode, // 关闭忽略HTML标签，即开启识别HTML标签，默认为false
                smartLists: true,
                smartypants: true,
            };
            return editormd.$marked(markdown, markedOptions);
        };
        this._height = '530px';
        this._width = '100%';
        this._enableMathjax = false;
        this._zIndex = 1000;
        this._placeholder = '';
        this._realTimePreviewCache = undefined;
        this.$cmContentChanged = new BehaviorSubject('');
        this._sub = new Subject();
        this.onChange = (_) => { };
        this.onTouch = () => { };
        /**
         * override editormd.save()
         */
        this.save = () => {
            const render = (html) => {
                this.previewContainerEl.innerHTML = html;
                if (this._enableMathjax) {
                    // 确保mathjax得到了加载
                    this.loadMathjax().then(_ => {
                        MathJax.Hub.Queue(['Typeset', MathJax.Hub, this.previewContainerEl]);
                    });
                }
                if (this.highlightCodeOption.enable && this.highlightCodeOption.highlightCode) {
                    this.highlightCodeOption.highlightCode(this.previewFrame);
                }
                this.previewContentReady.emit(this);
            };
            if (this.editorState.preview || this.editorState.watching) {
                if (!this._markdownToHtml) {
                    return;
                }
                let htmlOrPromise = this._markdownToHtml(this.markdown);
                if (htmlOrPromise instanceof Promise) {
                    htmlOrPromise.then(result => {
                        render(result);
                    });
                }
                else {
                    render(htmlOrPromise);
                }
            }
            this.markdownTextarea.textContent = this.markdown;
            return this._editor;
        };
    }
    get markdown() {
        return this._editor?.getMarkdown() ?? '';
    }
    /**
     * 编辑器当前内容的总行数
     */
    get lineCount() {
        return this._editor?.cm.lineCount() ?? 0;
    }
    /**
     * 关闭实时预览
     *
     */
    closeRealtimePreview() {
        this.execEnsureEditormdLoaded(editor => {
            editor.unwatch();
            editor.cm.refresh();
        });
    }
    /**
     * 打开实时预览
     *
     */
    openRealtimePreview() {
        this.execEnsureEditormdLoaded(editor => {
            editor.watch();
            editor.cm.refresh();
        });
    }
    /**
     * 设置编辑器焦点
     *
     * @param {boolean} [moveCursorToEnd=false] 是否将光标移动到最后
     */
    focusEditor(moveCursorToEnd = false) {
        this.execEnsureEditormdLoaded(editor => {
            editor.focus();
            if (moveCursorToEnd) {
                editor.setCursor({
                    line: this.lineCount,
                    ch: 0,
                });
            }
        });
    }
    /**
     * 在当前光标位置插入文本值
     *
     * @param {string} value
     */
    insertValue(value) {
        this.execEnsureEditormdLoaded(editor => {
            editor.insertValue(value);
        });
    }
    /**
     * 替换 from 和 to 直接的内容为 replacement
     *
     * @param {string} replacement
     * @param {Cursor} from
     * @param {Cursor} to
     * @param {string} origin
     */
    replaceRange(replacement, from, to, origin) {
        this.execEnsureEditormdLoaded(editor => {
            editor.cm.replaceRange(replacement, from, to, origin);
        });
    }
    /**
     * 退出全屏
     *
     */
    exitFullScreen() {
        this.execEnsureEditormdLoaded(editor => {
            editor.fullscreenExit();
            this.resize();
        });
    }
    /**
     * 进入全屏编辑模式
     *
     */
    enterFullScreen() {
        this.execEnsureEditormdLoaded(editor => {
            editor.fullscreen();
            this.resize();
        });
    }
    ngOnInit() { }
    async ngAfterViewInit() {
        await this.initEditormd();
    }
    ngOnDestroy() {
        this.initializing$.complete();
        if (componentInstances[this.id]) {
            delete componentInstances[this.id];
        }
        this._sub.next();
        this._sub.complete();
    }
    writeValue(value) {
        if (typeof value === 'string' && !!value) {
            this.execEnsureEditormdLoaded(editor => {
                editor.setValue(value);
            });
        }
    }
    registerOnChange(fn) {
        this.onChange = fn;
    }
    registerOnTouched(fn) {
        this.onTouch = fn;
    }
    setDisabledState(isDisabled) {
        Array.from(document.querySelectorAll('emd-markdown-editor textarea'))
            .map(x => x)
            .filter(x => !!x)
            .forEach(item => {
            item.disabled = isDisabled;
            if (isDisabled) {
                item.blur();
            }
        });
    }
    onFocus() {
        setTimeout(() => {
            this.setDisabledState(this.disabled);
        }, 20);
    }
    /**
     * 剪贴板事件处理程序
     */
    async onPaste(event) {
        if (event) {
            await this.uploadImageFromClipboard(event);
        }
    }
    onDrop(e) {
        e.preventDefault();
        e.stopPropagation();
        const file = Array.from(e.dataTransfer?.files ?? [])
            .filter(x => x.type.startsWith('image/'))
            .pop();
        if (file) {
            this.uploadImageAndInsertImageLinkToEditor(file);
        }
    }
    allowDrop(ev) {
        ev.preventDefault();
        ev.stopPropagation();
    }
    syncPreviewScroll() {
        if (!this.realtimePreview)
            return;
        this.execEnsureEditormdLoaded(editor => {
            const cm = editor.cm;
            const scrollEl = cm.getScrollerElement();
            scrollEl.dispatchEvent(new Event('scroll', { bubbles: true }));
        });
    }
    /**
     * 上传剪贴板中的图片文件
     *
     * @private
     * @param event
     */
    async uploadImageFromClipboard(event) {
        const clipboardData = event.clipboardData;
        if (!clipboardData) {
            return;
        }
        const lastClipboardIndex = (clipboardData.items?.length ?? 0) - 1;
        if (lastClipboardIndex >= 0 &&
            clipboardData.items[lastClipboardIndex].kind === 'file' &&
            clipboardData.items[lastClipboardIndex].type.startsWith('image/')) {
            const image = clipboardData.items[lastClipboardIndex].getAsFile();
            if (!image) {
                return;
            }
            event.preventDefault();
            this.uploadImageAndInsertImageLinkToEditor(image);
        }
    }
    uploadImageFromFile() {
        const inputEl = document.createElement('input');
        inputEl.type = 'file';
        inputEl.accept = 'image/*';
        inputEl.click();
        inputEl.onchange = () => {
            const files = Array.from(inputEl.files ?? []);
            if (files.length === 1) {
                this.uploadImageAndInsertImageLinkToEditor(files[0]);
            }
            else {
                console.error('No image file selected to upload, image upload flow canceled');
            }
        };
    }
    async uploadImageAndInsertImageLinkToEditor(image) {
        let generatedImageUrl = '';
        const uploadingTips = '![image](uploading...)';
        this.insertValue(uploadingTips);
        const cursor = this._editor.getCursor();
        try {
            generatedImageUrl = await this._uploadImage(image);
        }
        catch (err) {
            console.warn(`paste to update image failed, ${image.name}`, err);
        }
        const succeed = generatedImageUrl && generatedImageUrl.match(/^https?:\/\//);
        this.replaceRange(succeed ? `![image](${generatedImageUrl})\n` : '', {
            line: cursor.line,
            ch: cursor.ch - uploadingTips.length,
        }, {
            line: cursor.line,
            ch: cursor.ch,
        }, uploadingTips);
    }
    /**
     * 初始化编辑器
     * 1. 动态加载资源外部(包括css和js)
     * 2. 初始化编辑器设置, 并对相关事件进行监听
     *
     * @private
     */
    async initEditormd() {
        // 防止 this 失指
        componentInstances[this.id] = this;
        this.initializing$.next(true);
        // 加载依赖
        await this.loadJquery(); // 先加载jquery
        if (!(await waitUntilJQueryLoaded())) {
            console.error('等待jQuery加载超时!');
            this.initializing$.next(false);
            return;
        }
        await Promise.all([this.loadEditormdCss(), this.loadEditormd()]);
        if (!(await waitUntilEditormdLoaded())) {
            console.error('加载编辑器超时!');
            this.initializing$.next(false);
            return;
        }
        const toolbarIcons = this.buildToolbarIcons();
        editormd.version = this.editormdVersionSuffix;
        this._editor = editormd(this.id, {
            autoFocus: false,
            path: `${this.libPath}lib/`, // 资源所在文件夹路径
            height: this._height, // 高度
            width: this._width,
            saveHTMLToTextarea: true,
            toolbarIcons: toolbarIcons,
            toolbarIconsClass: {
                uploadImage: 'fa fa-picture-o',
                'cnb-search-replace': 'fa fa-search',
                save: 'fa fa-save',
            },
            toolbarCustomIcons: {
                'cnb-insert-collapsible-code-block': `<a href="javascript:;" title="可折叠代码段" unselectable="on" ><i class="fa" name="cnb-insert-collapsible-code-block" ><svg name="cnb-insert-collapsible-code-block" style="fill: currentColor;" class="icon" viewBox="0 0 1024 1024" xmlns="http://www.w3.org/2000/svg"><path d="M952 612c4.4 0 8-3.6 8-8v-56c0-4.4-3.6-8-8-8H298c-14.2-35.2-48.7-60-89-60-53 0-96 43-96 96s43 96 96 96c40.3 0 74.8-24.8 89-60h150.3v152c0 55.2 44.8 100 100 100H952c4.4 0 8-3.6 8-8v-56c0-4.4-3.6-8-8-8H548.3c-15.5 0-28-12.5-28-28V612H952zM451.7 313.7l172.5 136.2c6.3 5.1 15.8 0.5 15.8-7.7V344h264c4.4 0 8-3.6 8-8v-60c0-4.4-3.6-8-8-8H640v-98.2c0-8.1-9.4-12.8-15.8-7.7L451.7 298.3c-4.9 3.9-4.9 11.5 0 15.4z" p-id="1987"></path></svg></i></a>`,
                'task-lists': TaskListsPlugin.toolbarIcon,
                'exit-fullscreen': `<svg viewBox="0 0 1024 1024" fill="currentColor" xmlns="http://www.w3.org/2000/svg"><path d="M396.8 140.8c-12.8-4.266667-25.6-2.133333-34.133333 6.4l-76.8 76.8-128-125.866667c-17.066667-17.066667-42.666667-17.066667-59.733334 0-17.066667 17.066667-17.066667 42.666667 0 59.733334l125.866667 125.866666-76.8 76.8c-8.533333 8.533333-12.8 23.466667-6.4 34.133334 4.266667 12.8 17.066667 19.2 29.866667 19.2h213.333333c17.066667 0 32-14.933333 32-32V170.666667c0-12.8-8.533333-25.6-19.2-29.866667zM800 738.133333l76.8-76.8c8.533333-8.533333 12.8-23.466667 6.4-34.133333s-17.066667-19.2-29.866667-19.2H640c-17.066667 0-32 14.933333-32 32v213.333333c0 12.8 8.533333 25.6 19.2 29.866667 4.266667 2.133333 8.533333 2.133333 12.8 2.133333 8.533333 0 17.066667-4.266667 23.466667-8.533333l76.8-76.8 125.866666 125.866667c8.533333 8.533333 19.2 12.8 29.866667 12.8s21.333333-4.266667 29.866667-12.8c17.066667-17.066667 17.066667-42.666667 0-59.733334l-125.866667-128zM384 608H170.666667c-12.8 0-25.6 8.533333-29.866667 19.2-4.266667 12.8-2.133333 25.6 6.4 34.133333l76.8 76.8-125.866667 125.866667c-17.066667 17.066667-17.066667 42.666667 0 59.733333 8.533333 10.666667 19.2 14.933333 29.866667 14.933334s21.333333-4.266667 29.866667-12.8l125.866666-125.866667 76.8 76.8c6.4 6.4 14.933333 8.533333 23.466667 8.533333 4.266667 0 8.533333 0 12.8-2.133333 12.8-4.266667 19.2-17.066667 19.2-29.866667V640c0-17.066667-14.933333-32-32-32zM640 416h213.333333c12.8 0 25.6-8.533333 29.866667-19.2s2.133333-25.6-6.4-34.133333l-76.8-76.8 125.866667-125.866667c17.066667-17.066667 17.066667-42.666667 0-59.733333-17.066667-17.066667-42.666667-17.066667-59.733334 0l-125.866666 125.866666L663.466667 149.333333c-8.533333-8.533333-23.466667-12.8-34.133334-6.4-12.8 4.266667-19.2 17.066667-19.2 29.866667v213.333333c-2.133333 14.933333 12.8 29.866667 29.866667 29.866667z"></path></svg>`,
            },
            toolbarHandlers: {
                uploadImage: function () {
                    const comp = componentInstances[this.id];
                    comp.uploadImageFromFile();
                },
                'cnb-search-replace': function () {
                    const comp = componentInstances[this.id];
                    comp.openSearchDialog();
                },
                'cnb-insert-collapsible-code-block': function () {
                    const comp = componentInstances[this.id];
                    comp.insertCollapsibleCodeBlock();
                },
                'task-lists': function () {
                    const comp = componentInstances[this.id];
                    TaskListsPlugin.insertTaskLists(comp._editor);
                },
                save: function () {
                    const comp = componentInstances[this.id];
                    comp.toolbarIconSaveClicked.emit();
                },
            },
            htmlDecode: true,
            watch: false,
            matchWordHighlight: false,
            searchReplace: false,
            autoCloseTags: true,
            previewCodeHighlight: false,
            toolbar: true, // show or hide toolbar
            flowChart: false,
            sequenceDiagram: false,
            tex: false, // TeX(LaTeX), based on KaTeX
            emoji: false,
            atLink: false, // for @link
            emailLink: false, // for email address auto link
            taskList: false, // Enable Github Flavored Markdown task lists
            toolbarAutoFixed: false,
            placeholder: this._placeholder,
            delay: 500,
            toc: false, // Table of contents
            tocm: false, // Using [TOCM], auto create ToC dropdown menu
            tocTitle: '', // for ToC dropdown menu button
            tocDropdown: false, // Enable/disable Table Of Contents dropdown menu
            tocContainer: '', // Custom Table Of Contents Container Selector
            tocStartLevel: 1, // Said from H1 to create ToC,
            lineNumbers: this.lineNumbers,
            manualRefreshRealtimePreview: true,
            onpreviewing: function () {
                const comp = componentInstances[this.id];
                comp.loadPreviewStyle();
            },
            onwatch: function () {
                const comp = componentInstances[this.id];
                comp.loadPreviewStyle();
            },
            onfullscreen: function () {
                const comp = componentInstances[this.id];
                comp._realTimePreviewCache = comp.realtimePreview;
                comp.focusEditor(true);
                comp.openRealtimePreview();
                comp._zIndex++;
            },
            onfullscreenExit: function () {
                const comp = componentInstances[this.id];
                if (comp._realTimePreviewCache === false) {
                    comp.closeRealtimePreview();
                }
                comp._zIndex--;
            },
            onload: function () {
                const comp = componentInstances[this.id];
                comp.onEditormdLoaded();
            },
            onresize: function () { },
            lang: {
                // Language data, you can custom your language.
                name: 'zh-cn',
                description: 'powered by editormd',
                tocTitle: '目录',
                toolbar: {
                    'cnb-search-replace': '查找/替换',
                    uploadImage: '上传图片',
                    save: this.saveIconText,
                },
            },
        });
        this.initializing$
            .pipe(takeUntil(this._sub), filter(v => v === false))
            .subscribe(() => {
            this.$cmContentChanged.pipe(debounceTime(500), takeUntil(this._sub)).subscribe(v => {
                if (v) {
                    this.save();
                }
            });
            this.$cmContentChanged.pipe(takeUntil(this._sub), skip(1), debounceTime(200)).subscribe(() => {
                this.onTyped();
            });
        });
    }
    buildToolbarIcons() {
        return (this.toolbarIcons && this.toolbarIcons.length > 0
            ? this.toolbarIcons
            : defaultToolbarIcons.filter(i => i && (this.enableSearchReplace ? true : i !== 'cnb-search-replace'))).concat(...this.appendToolbarIcons);
    }
    async onEditormdLoaded() {
        this._editor.save = this.save;
        // bind the change event
        this._editor.cm.on('change', () => {
            this.$cmContentChanged.next(this.markdown);
        });
        this._editor.cm.on('focus', () => {
            this.onFocus();
        });
        this.initKeyMaps();
        this.initializing$.next(false);
        this.loaded.emit(this);
        if (this.realtimePreview) {
            this.openRealtimePreview();
        }
        else {
            this.closeRealtimePreview();
        }
    }
    initKeyMaps() {
        if (this.enableSearchReplace) {
            this._editor.cm.removeKeyMap('Ctrl-F');
            this._editor.cm.addKeyMap({
                'Cmd-F': () => this.openSearchDialog(),
                'Ctrl-F': () => this.openSearchDialog(),
            });
        }
    }
    /**
     * 加载并配置mathjax
     */
    async loadMathjax() {
        await loadScript({
            src: this._mathjaxSrc,
            id: 'mathjax-script',
            appendTo: this.previewDocument.head,
        });
        this.previewWindow.MathJax.Hub.Config({
            elements: [this.previewContainerEl.id],
            messageStyle: 'none',
            tex2jax: {
                inlineMath: [['\\(', '\\)']],
                displayMath: [['\\[', '\\]']],
                processClass: 'math',
                processEscapes: true,
            },
            TeX: {
                equationNumbers: {
                    autoNumber: 'AMS',
                    useLabelIds: true,
                },
                extensions: ['extpfeil.js'],
            },
            'HTML-CSS': { linebreaks: { automatic: true } },
            SVG: { linebreaks: { automatic: true } },
        });
    }
    /**
     * 确保在editormd初始化完成后执行指定的操作
     *
     * @private
     * @param {(editor: any) => void} action
     */
    execEnsureEditormdLoaded(action) {
        this.initializing$.subscribe(initializing => {
            if (!initializing) {
                action(this._editor);
            }
        });
    }
    /**
     * 加载jquery
     */
    async loadJquery() {
        if (window.$) {
            return;
        }
        await loadScript({
            src: this.jqueryOption.src,
            id: this.jqueryOption.tagId,
            executionStrategy: 'defer',
        });
    }
    /**
     * 加载editormd.min.js
     *
     * @private
     */
    async loadEditormd() {
        await loadScript({
            src: `${this.libPath}editormd.min.js?v=${this.editormdVersionSuffix}`,
            id: 'editormd-js',
            executionStrategy: 'defer',
        });
    }
    /**
     * 加载editormd.css
     *
     * @private
     */
    async loadEditormdCss() {
        await loadCss({
            href: `${this.libPath}css/editormd.min.css?v=${this.editormdVersionSuffix}`,
            id: 'editormd-css',
        });
    }
    /**
     * 发生输入事件时的处理程序
     *
     * @private
     */
    onTyped() {
        this.onChange(this.markdown);
        this.onTouch();
        this.input.emit(this.markdown);
    }
    resize(width, height) {
        this.execEnsureEditormdLoaded(editor => {
            editor.resize(width, height);
            this.refreshCodeMirror();
        });
    }
    /**
     * Docs from CodeMirror official web site:
     * If your code does something to change the size of the editor element (window resizes are
     * already listened for), or unhides it, you should probably follow up by calling this method to ensure CodeMirror is still
     * looking as intended.
     *
     * @private
     */
    refreshCodeMirror() {
        this.execEnsureEditormdLoaded(editor => {
            if (editor && editor.cm) {
                editor.cm.refresh();
            }
            else {
                console.warn('Failed to invoke CodeMirror refresh method, the reference is not set to the instance of CodeMirror');
            }
        });
    }
    openSearchDialog() {
        this._editor.executePlugin('cnbSearchReplaceDialog', 'cnb-search-replace-dialog/cnb-search-replace-dialog');
    }
    insertCollapsibleCodeBlock() {
        this._editor.executePlugin('cnbInsertCollapsibleCodeBlock', 'cnb-insert-collapsible-code-block/cnb-insert-collapsible-code-block');
    }
    loadPreviewStyle() {
        return loadCss({
            href: `/${this.libPath}css/editormd.min.css`,
            appendTo: this.previewDocument.head,
            id: 'editormd-style',
        }).then();
    }
    onWindowResize() {
        this.resize();
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "17.3.0", ngImport: i0, type: EditormdComponent, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "17.3.0", type: EditormdComponent, selector: "emd-markdown-editor", inputs: { libPath: "libPath", lineNumbers: "lineNumbers", jqueryOption: "jqueryOption", toolbarIcons: "toolbarIcons", appendToolbarIcons: "appendToolbarIcons", saveIconText: "saveIconText", editormdVersionSuffix: "editormdVersionSuffix", disabled: "disabled", highlightCodeOption: "highlightCodeOption", mathjaxSrc: "mathjaxSrc", uploadImage: "uploadImage", enableSearchReplace: "enableSearchReplace", markdownToHtml: "markdownToHtml", loadingTemplate: "loadingTemplate", height: "height", width: "width", enableMathjax: "enableMathjax", zIndex: "zIndex", realtimePreview: "realtimePreview", placeholder: "placeholder" }, outputs: { input: "input", toolbarIconSaveClicked: "toolbarIconSaveClicked", previewContentReady: "previewContentReady", loaded: "loaded" }, host: { listeners: { "window:resize": "onWindowResize()" } }, providers: [
            {
                provide: NG_VALUE_ACCESSOR,
                useExisting: forwardRef(() => EditormdComponent),
                multi: true,
            },
        ], ngImport: i0, template: "<div [id]=\"id\"\n     class=\"ng-editormd-container\"\n     [style.z-index]=\"zIndex\"\n     [style.height]=\"height\"\n     [style.width]=\"width\"\n     (paste)=\"onPaste($event)\"\n     (dragover)=\"allowDrop($event)\"\n     (drop)=\"onDrop($event)\">\n    <ng-container *ngIf=\"initializing$|async\">\n        <div style=\"width: 100%; position: relative; background: transparent;\"\n             [style.height]=\"height\"\n             [style.z-index]=\"zIndex+1\">\n            <div class=\"loading-mask\">\n                <div class=\"loading\">\n                    <ng-template [ngIf]=\"!loadingTemplate\"\n                                 [ngIfElse]=\"loadingTemplate\">\n                        \u7F16\u8F91\u5668\u52A0\u8F7D\u4E2D...\n                    </ng-template>\n                </div>\n            </div>\n        </div>\n    </ng-container>\n    <ng-container *ngIf=\"disabled\">\n        <div style=\"width: 100%; position: absolute; top: 0; bottom: 0; background: transparent;\"\n             [style.height]=\"height\"\n             [style.z-index]=\"zIndex+1\">\n            <div class=\"loading-mask\">\n                <div class=\"loading\"></div>\n            </div>\n        </div>\n    </ng-container>\n    <textarea style=\"display: none;\"></textarea>\n</div>\n", styles: ["::ng-deep .editormd-dialog-mask-con,::ng-deep .editormd-container-mask{background:none!important}.loading-mask{background:#fffc;position:absolute;width:100%;height:100%;inset:0}.loading{display:flex;width:100%;height:100%;justify-content:center;align-items:center;-webkit-user-select:none;user-select:none}::ng-deep .editormd-dialog.editormd-cnb-search-replace .editormd-form input[type=text]{width:285px}::ng-deep .editormd-toolbar li a i.fa.fa-table{transform:translateY(1px)}\n"], dependencies: [{ kind: "directive", type: i1.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }, { kind: "pipe", type: i1.AsyncPipe, name: "async" }] }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "17.3.0", ngImport: i0, type: EditormdComponent, decorators: [{
            type: Component,
            args: [{ selector: 'emd-markdown-editor', providers: [
                        {
                            provide: NG_VALUE_ACCESSOR,
                            useExisting: forwardRef(() => EditormdComponent),
                            multi: true,
                        },
                    ], template: "<div [id]=\"id\"\n     class=\"ng-editormd-container\"\n     [style.z-index]=\"zIndex\"\n     [style.height]=\"height\"\n     [style.width]=\"width\"\n     (paste)=\"onPaste($event)\"\n     (dragover)=\"allowDrop($event)\"\n     (drop)=\"onDrop($event)\">\n    <ng-container *ngIf=\"initializing$|async\">\n        <div style=\"width: 100%; position: relative; background: transparent;\"\n             [style.height]=\"height\"\n             [style.z-index]=\"zIndex+1\">\n            <div class=\"loading-mask\">\n                <div class=\"loading\">\n                    <ng-template [ngIf]=\"!loadingTemplate\"\n                                 [ngIfElse]=\"loadingTemplate\">\n                        \u7F16\u8F91\u5668\u52A0\u8F7D\u4E2D...\n                    </ng-template>\n                </div>\n            </div>\n        </div>\n    </ng-container>\n    <ng-container *ngIf=\"disabled\">\n        <div style=\"width: 100%; position: absolute; top: 0; bottom: 0; background: transparent;\"\n             [style.height]=\"height\"\n             [style.z-index]=\"zIndex+1\">\n            <div class=\"loading-mask\">\n                <div class=\"loading\"></div>\n            </div>\n        </div>\n    </ng-container>\n    <textarea style=\"display: none;\"></textarea>\n</div>\n", styles: ["::ng-deep .editormd-dialog-mask-con,::ng-deep .editormd-container-mask{background:none!important}.loading-mask{background:#fffc;position:absolute;width:100%;height:100%;inset:0}.loading{display:flex;width:100%;height:100%;justify-content:center;align-items:center;-webkit-user-select:none;user-select:none}::ng-deep .editormd-dialog.editormd-cnb-search-replace .editormd-form input[type=text]{width:285px}::ng-deep .editormd-toolbar li a i.fa.fa-table{transform:translateY(1px)}\n"] }]
        }], ctorParameters: () => [], propDecorators: { input: [{
                type: Output
            }], toolbarIconSaveClicked: [{
                type: Output
            }], previewContentReady: [{
                type: Output
            }], loaded: [{
                type: Output
            }], libPath: [{
                type: Input
            }], lineNumbers: [{
                type: Input
            }], jqueryOption: [{
                type: Input
            }], toolbarIcons: [{
                type: Input
            }], appendToolbarIcons: [{
                type: Input
            }], saveIconText: [{
                type: Input
            }], editormdVersionSuffix: [{
                type: Input
            }], disabled: [{
                type: Input
            }], highlightCodeOption: [{
                type: Input
            }], mathjaxSrc: [{
                type: Input
            }], uploadImage: [{
                type: Input
            }], enableSearchReplace: [{
                type: Input
            }], markdownToHtml: [{
                type: Input
            }], loadingTemplate: [{
                type: Input
            }], height: [{
                type: Input,
                args: ['height']
            }], width: [{
                type: Input,
                args: ['width']
            }], enableMathjax: [{
                type: Input
            }], zIndex: [{
                type: Input,
                args: ['zIndex']
            }], realtimePreview: [{
                type: Input
            }], placeholder: [{
                type: Input
            }], onWindowResize: [{
                type: HostListener,
                args: ['window:resize']
            }] } });

class EditormdModule {
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "17.3.0", ngImport: i0, type: EditormdModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule }); }
    static { this.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "17.3.0", ngImport: i0, type: EditormdModule, declarations: [EditormdComponent], imports: [CommonModule], exports: [EditormdComponent] }); }
    static { this.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "17.3.0", ngImport: i0, type: EditormdModule, imports: [CommonModule] }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "17.3.0", ngImport: i0, type: EditormdModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [EditormdComponent],
                    imports: [CommonModule],
                    exports: [EditormdComponent],
                }]
        }] });

/*
 * Public API Surface of ng-editormd
 */

/**
 * Generated bundle index. Do not edit.
 */

export { EditormdComponent, EditormdModule };
//# sourceMappingURL=cnblogs-ng-editormd.mjs.map
